xquery version "3.1";

module namespace tclientt="https://sade.textgrid.de/ns/client/tests";

import module namespace functx="http://www.functx.com";
import module namespace client="https://sade.textgrid.de/ns/client" at "../modules/client.xqm";
import module namespace test="http://exist-db.org/xquery/xqsuite" at "resource:org/exist/xquery/lib/xqsuite/xqsuite.xql";

declare namespace sparql-results="http://www.w3.org/2005/sparql-results#";
declare namespace tgmd="http://textgrid.info/namespaces/metadata/core/2010";


declare
    %test:args("1a2b3.4", "https://textgridlab.org/1.0/tgcrud/rest", "1234") %test:assertEquals("https://textgridlab.org/1.0/tgcrud/rest/1a2b3.4/metadata?sessionId=1234")
    %test:args("1a2b3.4", "https://textgridlab.org/1.0/tgcrud/rest", "") %test:assertEquals("https://textgridlab.org/1.0/tgcrud/rest/1a2b3.4/metadata?sessionId=")
function tclientt:make-metadata-request-URL($id as xs:string,
    $tgcrud-url as xs:string,
    $sid as xs:string)
as xs:string {
    client:make-metadata-request-URL($id, $tgcrud-url, $sid)
};

declare
    %test:args("1a2b3.4", "https://textgridlab.org/1.0/tgcrud/rest", "1234") %test:assertEquals("https://textgridlab.org/1.0/tgcrud/rest/1a2b3.4/data?sessionId=1234")
    %test:args("1a2b3.4", "https://textgridlab.org/1.0/tgcrud/rest", "") %test:assertEquals("https://textgridlab.org/1.0/tgcrud/rest/1a2b3.4/data?sessionId=")
function tclientt:make-data-request-URL($id as xs:string,
    $tgcrud-url as xs:string,
    $sid as xs:string)
as xs:string {
    client:make-data-request-URL($id, $tgcrud-url, $sid)
};


declare
    %test:args("textgrid:jmzc", "https://textgridlab.org/1.0/tgcrud/rest", "") %test:assertXPath("$result//text() = 'Auf das schöne Geschlecht'")
    %test:pending %test:assumeInternetAccess("https://textgridlab.org/1.0/tgcrud/rest/")
function tclientt:get-metadata($id as xs:string,
    $tgcrud-url as xs:string,
    $sid as xs:string?)
as element(tgmd:MetadataContainerType) {
    client:get-metadata($id, $tgcrud-url, $sid)
    => local:fix-namespace()
};

declare
    %test:args("jmzc", "https://textgridlab.org/1.0/tgcrud/rest", "")
    %test:assertError("client02")
    %test:pending %test:assumeInternetAccess("https://textgridlab.org/1.0/tgcrud/rest/")
function tclientt:get-metadata-error($id as xs:string,
    $tgcrud-url as xs:string,
    $sid as xs:string?)
as node() {
    client:get-metadata($id, $tgcrud-url, $sid)
    => local:fix-namespace()
};


declare
    %test:args("https://textgridlab.org/1.0/tgcrud/rest/textgrid:jmzc/metadata?sessionId=", "textgrid:jmzc") %test:assertXPath("$result//text() = 'Auf das schöne Geschlecht'")
    %test:pending %test:assumeInternetAccess("https://textgridlab.org/1.0/tgcrud/rest/")
function tclientt:send-metadata-request($request-url as xs:string,
    $id as xs:string)
as node() {
    client:send-metadata-request($request-url, $id)
    => local:fix-namespace()
};


declare
    %test:args("https://textgridlab.org/1.0/tgcrud/rest/textgrid:qwerty/metadata?sessionId=", "textgrid:qwerty")
    %test:assertError("client02")
    %test:pending %test:assumeInternetAccess("https://textgridlab.org/1.0/tgcrud/rest/")
function tclientt:send-metadata-request-error($request-url as xs:string,
    $id as xs:string)
as node() {
    client:send-metadata-request($request-url, $id)
    => local:fix-namespace()
};

declare
    %test:args("1234") %test:assertTrue
function tclientt:make-sparql-query-for-aggregated-uris($tguri as xs:string)
as xs:boolean {
    let $query := client:make-sparql-query-for-aggregated-uris($tguri)
    let $proof-read-query :=
        "PREFIX ore:<http://www.openarchives.org/ore/terms/> PREFIX tg:<http://textgrid.info/relation-ns#> SELECT ?s WHERE { <1234> (ore:aggregates/tg:isBaseUriOf|ore:aggregates)* ?s. FILTER not exists { ?s tg:isDeleted true } . }"
    return $query = $proof-read-query
};

declare
  %test:args("textgrid:vv6f.0", "https://textgridlab.org/1.0/triplestore/textgrid-public")
  %test:assertExists
  %test:assertEquals("textgrid:vv6f", "textgrid:vv6g", "textgrid:vvc3")
  %test:assertXPath("count($result) eq 3")
  %test:pending %test:assumeInternetAccess("https://textgridlab.org/1.0/triplestore/textgrid-public")
function tclientt:get-latest-aggregated-uris($tguri as xs:string,
    $rdfstore as xs:string)
as xs:string* {
    client:get-latest-aggregated-uris($tguri, $rdfstore)
};

declare
    %test:args("textgrid:vv6f.0", "https://textgridlab.org/1.0/triplestore/textgrid-public")
    %test:assertXPath("$result//text() = 'textgrid:vv6f.0'")
    %test:pending %test:assumeInternetAccess("https://textgridlab.org/1.0/triplestore/textgrid-public")
function tclientt:get-sparql-result($tguri as xs:string,
    $rdfstore as xs:string)
as element(sparql-results:sparql) {
    let $result := client:get-sparql-result($tguri, $rdfstore)
    return
        local:fix-namespace($result)
};




declare function local:fix-namespace($node as element())
as item() {
    let $namespace-uri := $node/namespace-uri()
    let $prefix :=
        switch ($namespace-uri)
            case "http://www.w3.org/2005/sparql-results#" return "sparql-results"
            case "http://textgrid.info/namespaces/metadata/core/2010" return "tgmd"
            default return ""
    return
        functx:change-element-ns($node, $namespace-uri, $prefix)
};

declare
    %test:assertEquals("textgrid:vv6f", "textgrid:vv6g", "textgrid:vvc3")
function tclientt:get-unrevisioned-aggregated-uris()
as xs:string+ {
    let $sparql-result :=
        <sparql xmlns="http://www.w3.org/2005/sparql-results#">
        	<head>
        		<variable name="s"/>
        	</head>
        	<results>
        		<result>
        			<binding name="s">
        				<uri>textgrid:vv6f.0</uri>
        			</binding>
        		</result>
        		<result>
        			<binding name="s">
        				<uri>textgrid:vv6g.0</uri>
        			</binding>
        		</result>
        		<result>
        			<binding name="s">
        				<uri>textgrid:vvc3</uri>
        			</binding>
        		</result>
        		<result>
        			<binding name="s">
        				<uri></uri>
        			</binding>
        		</result>
        	</results>
        </sparql>
    return
        client:get-unrevisioned-aggregated-uris($sparql-result)
};

declare
    %test:args("textgrid:1234") %test:assertEquals("1234")
    %test:args("textgrid:with:more:colons:1234") %test:assertEquals("withmorecolons1234")
    %test:args("no_prefix") %test:assertEquals("no_prefix")
function tclientt:remove-prefix($tguri as xs:string)
as xs:string {
    client:remove-prefix($tguri)
};

declare
    %test:args("PREFIX ore:<http://www.openarchives.org/ore/terms/>
    PREFIX tg:<http://textgrid.info/relation-ns#>
    
    SELECT ?s
        WHERE { <textgrid:vv6f.0> (ore:aggregates/tg:isBaseUriOf|ore:aggregates)* ?s.
        FILTER not exists { ?s tg:isDeleted true } . }",
        "https://textgridlab.org/1.0/triplestore/textgrid-public")
    %test:assertEquals("https://textgridlab.org/1.0/triplestore/textgrid-public?query=PREFIX%20ore%3A%3Chttp%3A%2F%2Fwww.openarchives.org%2Fore%2Fterms%2F%3E%0A%20%20%20%20PREFIX%20tg%3A%3Chttp%3A%2F%2Ftextgrid.info%2Frelation-ns%23%3E%0A%20%20%20%20%0A%20%20%20%20SELECT%20%3Fs%0A%20%20%20%20%20%20%20%20WHERE%20%7B%20%3Ctextgrid%3Avv6f.0%3E%20%28ore%3Aaggregates%2Ftg%3AisBaseUriOf%7Core%3Aaggregates%29%2A%20%3Fs.%0A%20%20%20%20%20%20%20%20FILTER%20not%20exists%20%7B%20%3Fs%20tg%3AisDeleted%20true%20%7D%20.%20%7D")
function tclientt:get-sparql-request-url($query as xs:string,
    $rdfstore as xs:string)
as xs:string {
    client:get-sparql-request-url($query, $rdfstore)
};


declare
    %test:args("textgrid:vv6f") %test:assertEquals(3)
    %test:args("textgrid:vvc3") %test:assertEmpty
function tclientt:find-max-rev($uri as xs:string)
as xs:integer? {
    let $sparql-result :=
        <sparql xmlns="http://www.w3.org/2005/sparql-results#">
        	<results>
        		<result>
        			<binding name="s">
        				<uri>textgrid:vv6f.0</uri>
        			</binding>
        		</result>
        		<result>
        			<binding name="s">
        				<uri>textgrid:vv6f.2</uri>
        			</binding>
        		</result>
        		<result>
        			<binding name="s">
        				<uri>textgrid:vv6f.3</uri>
        			</binding>
        		</result>
        		<result>
        			<binding name="s">
        				<uri>textgrid:vv6g.0</uri>
        			</binding>
        		</result>
        		<result>
        			<binding name="s">
        				<uri>textgrid:vvc3</uri>
        			</binding>
        		</result>
        	</results>
        </sparql>
    return
        client:find-max-rev($sparql-result, $uri)
};


declare
    %test:assertXPath("$result = 0 and $result = 1")
function tclientt:get-revisions-from-uris-mixed-revisions-0-1() {
    let $uris := ("textgrid:1234.0", "textgrid:1234.1", "textgrid:1234")
    return
        client:get-revisions-from-uris($uris)
};

declare
    %test:assertEmpty
function tclientt:get-revisions-from-uris-mixed-revisions() {
    let $uris := "textgrid:1234"
    return
        client:get-revisions-from-uris($uris)
};

declare
    %test:assertXPath("$result = 0 and $result = 1 and $result = 2")
function tclientt:get-revisions-from-uris-all-revisions() {
    let $uris := ("textgrid:1234.0", "textgrid:1234.1", "textgrid:1234.2")
    return
        client:get-revisions-from-uris($uris)
};

declare
    %test:args("https://textgridlab.org/1.0/triplestore/textgrid-public") %test:assertXPath("$result//sparql-results:uri")
function tclientt:sparql($rdfstore as xs:string)
as element(sparql-results:sparql) {
    let $query :=
        "PREFIX ore:<http://www.openarchives.org/ore/terms/>
        PREFIX tg:<http://textgrid.info/relation-ns#>
    
        SELECT ?s
            WHERE { <textgrid:vv6f.0> (ore:aggregates/tg:isBaseUriOf|ore:aggregates)* ?s.
            FILTER not exists { ?s tg:isDeleted true } . }"
    return
        client:sparql($query, $rdfstore)
        => local:fix-namespace()
};

declare
    %test:args("textgrid:vv6g", "https://textgridlab.org/1.0/tgcrud/rest/", "") %test:assertXPath("$result//*[local-name(.) = 'title'] = 'Biographie: Strachwitz, Moritz von'")
function tclientt:get-data($id as xs:string,
    $tgcrud-url as xs:string,
    $sid as xs:string?)
as item() {
    client:get-data($id, $tgcrud-url, $sid)//*[local-name(.) = "TEI"]
    => local:fix-namespace()
};

declare
    %test:args("textgrid:vv6g", "https://textgridlab.org/1.0/tgcrud/rest/", "") %test:assertXPath("$result/@status = '200' ")
function tclientt:get-data-request-header($id as xs:string,
    $tgcrud-url as xs:string,
    $sid as xs:string?)
as item() {
    client:get-data-request-header($id, $tgcrud-url, $sid)
};

declare
    %test:args("https://textgridlab.org/1.0/tgcrud/rest/textgrid:jkj1/data?sessionId=")
    %test:assertXPath("count($result) = 2")
function tclientt:make-request($request-url as xs:string) {
    client:make-request($request-url)
};

declare
    %test:args("https://textgridlab.org/1.0/tgcrud/rest/textgrid:qwerty1234/data?sessionId=")
    %test:assertError
function tclientt:make-request-error($request-url as xs:string) {
    client:make-request($request-url)
};

declare
    %test:args("textgrid:vqmz", "https://textgridlab.org/1.0/tgcrud/rest", "") %test:assertXPath("$result[1]/@status = '200'")
    %test:args("textgrid:12345asdf", "https://textgridlab.org/1.0/tgcrud/rest", "") %test:assertXPath("$result[1]/@status = '404'")
function tclientt:get-data-request-objects($id as xs:string,
    $tgcrud-url as xs:string,
    $sid as xs:string?)
as node()+ {
    client:get-data-request-objects($id, $tgcrud-url, $sid)
};

declare
    %test:assertTrue
function tclientt:has-one-of-uris-revision-all-revisions()
as xs:boolean {
    let $uris := ("textgrid:1234.1", "textgrid:1234.2", "textgrid:1234.3")
    return
        client:has-one-of-uris-revision($uris)
};

declare
    %test:assertTrue
function tclientt:has-one-of-uris-revision-mixed()
as xs:boolean {
    let $uris := ("textgrid:1234", "textgrid:1234.2", "textgrid:1234.3")
    return
        client:has-one-of-uris-revision($uris)
};

declare
    %test:assertFalse
function tclientt:has-one-of-uris-revision-no-revisions()
as xs:boolean {
    let $uris := "textgrid:1234"
    return
        client:has-one-of-uris-revision($uris)
};

declare
    %test:assertXPath("$result = 'textgrid:1234.2' and $result = 'textgrid:1234.3'")
function tclientt:get-uris-with-revision() {
    let $uris := ("textgrid:1234", "textgrid:1234.2", "textgrid:1234.3")
    return
        client:get-uris-with-revision($uris)
};
